/**
 * Created by Fausto Lelli <fausto@domotz.com> on 23/08/19.
 */

var DEFAULT_REMOTE_PAWN_DIR = '/opt/domotz/lib/node_modules/domotz-remote-pawn'

var generateOpenvpn = function (message, resourceLocator) {
    var path = resourceLocator.path;
    var lodash = resourceLocator.lodash;
    var cmdUtils = resourceLocator.utils.command;
    var myConsole = resourceLocator.log.decorateLogs("OPENVPN - ");
    var fileUtils = resourceLocator.utils.file;
    var childProcess = resourceLocator.utils.childProcess;
    var NEW_CLIENT_TIMEOUT = 28000;
    var outputFile = path.join(process.env.DOMOTZ_CONF_DIR, 'openvpn/newclient.ovpn');
    var script = path.join(process.env.DOMOTZ_REMOTE_PAWN_DIR || DEFAULT_REMOTE_PAWN_DIR, 'openvpn/domotzvpn.sh');

    return {
        describe: function () {
            return message.correlationId + " - generateVpnConfig - ";
        },
        execute: function (callback) {
            myConsole.debug("execute - GENERATE VPN CONFIG: " + " parameters are " + JSON.stringify(message.payload));

            var params = ['newclient']

            if (lodash.get(message, 'payload.routing_policy', '') === 'local') {
                params.push('no-redirect-gw');
            }

            var newClient = childProcess.spawn(script, params, {
                timeout: NEW_CLIENT_TIMEOUT,
                shell: true
            });

            newClient.on('exit', function (code) {
                if (code != 0) {
                    myConsole.info("execute - ERROR");
                    callback(cmdUtils.getResponse(null));
                } else {
                    var content = fileUtils.readFile(outputFile);
                    if (!content) {
                        callback(cmdUtils.getResponse(null, "unable to read openvpn output file"));
                        return;
                    }
                    myConsole.debug("execute - SENDING DATA");
                    callback(content.toString('base64'), null, {
                        asBinary: true,
                        'content-type': 'plain/text'
                    });
                }
                console.log('child process exited with code ' + code);
            });
        }
    };
};

var openvpnCheck = function (path, proc, logManager, fileManager, q) {
    var myConsole = logManager.decorateLogs("OPENVPN_CHECK - ");
    var versionStringPattern = "DOMOTZ_VPN_SERVER=OPENVPN";
    var script = path.join(process.env.DOMOTZ_REMOTE_PAWN_DIR || DEFAULT_REMOTE_PAWN_DIR, 'openvpn/domotzvpn.sh');
    var version = null;
    var RECONF_TIMEOUT = 50000;
    var CHECK_VERSION_TIMEOUT = 5000;

    function startConfiguration() {
        proc.spawn('sh', [script, 'reconf'], {timeout: RECONF_TIMEOUT}).then(
            function (output) {
                myConsole.info("configured" + output);
            }
        ).catch(function (error) {
            myConsole.info("configuration failed " + error);
            version = false;
        });
    }

    function getVersion() {
        if (version === false) {
            return q.reject();
        }
        if (version) {
            return q.resolve(version);
        }

        var deferred = q.defer();
        proc.spawn('sh', [script, 'version'], {timeout: CHECK_VERSION_TIMEOUT}).then(
            function (output) {
                myConsole.info("openvpn is available " + output);

                version = false;
                if (output) {
                    var lines = output.toUpperCase().split('\n');
                    var version_string = lines.filter(function (line) {
                        if (line) {
                            return line.startsWith(versionStringPattern);
                        }

                        return false;
                    });
                    if (version_string.length > 0) {
                        version = version_string[0].replace(versionStringPattern, "").trim();
                        deferred.resolve(version);
                    }
                    deferred.reject();
                } else {
                    deferred.reject();
                }
            }
        ).catch(function (error) {
            myConsole.info("openpvn is not available " + error);
            version = false;
            deferred.reject();
        });

        return deferred.promise;
    }

    return {
        getVersion: getVersion,
        startConfiguration: startConfiguration
    };
};

module.exports.command = generateOpenvpn;
module.exports.factory = openvpnCheck;